using System;
using cz.msdn.Piskvorky.Definice;

namespace cz.msdn.Piskvorky.TestovaciAplikace
{

	/// <summary>
	/// Tda HraciPlocha implementuje hraci plochu - pole kamenu a souvisejici operace
	/// </summary>
	public class HraciPlocha
	{
		/// <summary>
		/// Velikost tvercov hrac plochy (19)
		/// </summary>
		public const int VelikostPlochy=19;

		/// <summary>
		/// Poet kamen v ad, kter jsou teba k vtzstv ve he (5)
		/// </summary>
		public const int PocetVRadeKVitezstvi=5;

		private BarvaKamene[,] hraciPole=new BarvaKamene[VelikostPlochy,VelikostPlochy];
		private SouradnicePole[] viteznaRadaPoli;

		/// <summary>
		/// Hrac pole jako dvourozmrn pole typu BarvaKamene
		/// </summary>
		public BarvaKamene[,] HraciPole
		{
			get
			{
				//Nevracme pmo pole, ale pouze kopii - pole je intern zleitost a nen mon ho zven mnit
				return (BarvaKamene[,]) hraciPole.Clone();
			}
		}

		/// <summary>
		/// Umstn hracho kamene na hrac plochu.
		/// </summary>
		/// <param name="souradnice">Souadnice, kam bude kmen umstn</param>
		/// <param name="barva">Barva kamene, kter je teba umstit</param>
		/// <returns>Vrac true, pokud bylo pole neobsazen a kmen byl umstn, jinak false</returns>
		public bool UmistiKamen(SouradnicePole souradnice, BarvaKamene barva)
		{
			if ((hraciPole[souradnice.Radek,souradnice.Sloupec]!=BarvaKamene.Zadny) || JeVitez)
				return false;
			else
			{
				hraciPole[souradnice.Radek,souradnice.Sloupec]=barva;
				ZkusNajitViteznouRaduPoli();
				return true;
			}
		}

		private void ZkusNajitViteznouRaduPoli()
		{			
			viteznaRadaPoli = null;
			ZkusNajitPrimeVitezneRadyPoli(TransformaceDesky.zadna);
			if(viteznaRadaPoli==null)
				ZkusNajitPrimeVitezneRadyPoli(TransformaceDesky.symetrie);
			if(viteznaRadaPoli==null)
				ZkusNajitSikmeVitezneRadyPoli(TransformaceDesky.zadna);
			if(viteznaRadaPoli==null)
				ZkusNajitSikmeVitezneRadyPoli(TransformaceDesky.rotace);
		}


		/// <summary>
		/// Zjitn, zda m ji partie vtze. Vrt true, pokud existuje souvisl ada 'PocetVRadeKVitezstvi', jinak false.
		/// </summary>
		public bool JeVitez
		{
			get
			{
				return (viteznaRadaPoli != null);
			}
		}


		/// <summary>
		/// Zjitn, kdo v partii vyhrl. Pokud je vtz, vrt jeho barvu kamen, jinak vrt Zadny
		/// </summary>
		public BarvaKamene ViteznyHrac
		{
			get
			{
				if(viteznaRadaPoli==null)
					return BarvaKamene.Zadny;
				else
					return hraciPole[viteznaRadaPoli[0].Radek,viteznaRadaPoli[0].Sloupec];
			}
		}


		/// <summary>
		/// Zjitn vtzn ady kamen v tto he
		/// </summary>
		public SouradnicePole[] ViteznaRadaPoli
		{
			get
			{
				return (SouradnicePole[]) viteznaRadaPoli.Clone();
			}
		}

		#region Implementan detaily
		private enum TransformaceDesky {zadna=0, symetrie=1, rotace=2}
		private BarvaKamene HraciPoleZarazka(int radek, int sloupec, TransformaceDesky transformace)
		{
			int x=-1,y=-1;
			if	    (transformace== TransformaceDesky.zadna)
			{
				x=radek;y=sloupec;
			}
			else if (transformace == TransformaceDesky.symetrie)
			{
				x = sloupec; y=radek;
			}
			else if (transformace == TransformaceDesky.rotace)
			{
				x= (VelikostPlochy-1)-sloupec; y=radek;
			}
			
			if ((x>=0) && (x<HraciPlocha.VelikostPlochy) && (y>=0) && (y<HraciPlocha.VelikostPlochy))
				return hraciPole[x,y];
			else
				return BarvaKamene.Zadny;
		}

		private void ZkusNajitPrimeVitezneRadyPoli(TransformaceDesky transformace)
		{
			int rada=1;
			for(int y=0;y<HraciPlocha.VelikostPlochy;y++)
				for(int x=0;x<HraciPlocha.VelikostPlochy;x++)
				{
					if (HraciPoleZarazka(x,y,transformace)==BarvaKamene.Cerny && HraciPoleZarazka(x-1,y,transformace) == BarvaKamene.Cerny)
						rada++;
					else if(HraciPoleZarazka(x,y,transformace)==BarvaKamene.Bily && HraciPoleZarazka(x-1,y,transformace) == BarvaKamene.Bily)
						rada++;
					else if(HraciPoleZarazka(x,y,transformace)==BarvaKamene.Cerny && HraciPoleZarazka(x-1,y,transformace) != BarvaKamene.Cerny)
						rada=1;
					else if(HraciPoleZarazka(x,y,transformace)==BarvaKamene.Bily && HraciPoleZarazka(x-1,y,transformace) != BarvaKamene.Bily)
						rada=1;
					else 
						rada=0;
					if(rada==HraciPlocha.PocetVRadeKVitezstvi)	// vtzn n-tice nalezena
					{
						viteznaRadaPoli = new SouradnicePole[HraciPlocha.PocetVRadeKVitezstvi];
						for(int i=x-HraciPlocha.PocetVRadeKVitezstvi+1;i<=x;i++)
							if(transformace==TransformaceDesky.zadna)
								viteznaRadaPoli[i-(x-HraciPlocha.PocetVRadeKVitezstvi+1)] = new SouradnicePole(i,y);
							else	//TransformaceDesky.symetrie
								viteznaRadaPoli[i-(x-HraciPlocha.PocetVRadeKVitezstvi+1)] = new SouradnicePole(y,i);
						return;
					}
				}
		}

		private void ZkusNajitSikmeVitezneRadyPoli(TransformaceDesky transformace)
		{
			int rada=1;
			for(int diagonala=-HraciPlocha.VelikostPlochy+1;diagonala<=HraciPlocha.VelikostPlochy-1;diagonala++)
				for(int x=0;x<HraciPlocha.VelikostPlochy;x++)
				{
					if		(HraciPoleZarazka(x+diagonala,x,transformace)==BarvaKamene.Cerny && HraciPoleZarazka(x+diagonala-1,x-1,transformace) == BarvaKamene.Cerny)
						rada++;
					else if (HraciPoleZarazka(x+diagonala,x,transformace)==BarvaKamene.Bily && HraciPoleZarazka(x+diagonala-1,x-1,transformace) == BarvaKamene.Bily)
						rada++;
					else if (HraciPoleZarazka(x+diagonala,x,transformace)==BarvaKamene.Cerny && HraciPoleZarazka(x+diagonala-1,x-1,transformace) != BarvaKamene.Cerny)
						rada = 1;
					else if (HraciPoleZarazka(x+diagonala,x,transformace)==BarvaKamene.Bily && HraciPoleZarazka(x+diagonala-1,x-1,transformace) != BarvaKamene.Bily)
						rada = 1;
					else rada = 0;
					if(rada==HraciPlocha.PocetVRadeKVitezstvi)
					{
						viteznaRadaPoli = new SouradnicePole[HraciPlocha.PocetVRadeKVitezstvi];
						for(int i=x-HraciPlocha.PocetVRadeKVitezstvi+1;i<=x;i++)
							if(transformace == TransformaceDesky.zadna)
								viteznaRadaPoli[i-(x-HraciPlocha.PocetVRadeKVitezstvi+1)] = new SouradnicePole(i+diagonala,i);
							else			// TransformaceDesky.rotace
								viteznaRadaPoli[i-(x-HraciPlocha.PocetVRadeKVitezstvi+1)] = new SouradnicePole(VelikostPlochy-1-i,i+diagonala);
						return;
					}
				}
		}

		#endregion
	}
}
